/* Low-level functions for atomic operations.  AVR32 version.
   Copyright (C) 1997, 1998, 1999, 2000 Free Software Foundation, Inc.
   This file is part of the GNU C Library.

   The GNU C Library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   The GNU C Library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public
   License along with the GNU C Library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
   02111-1307 USA.  */

#ifndef _ATOMICITY_H
#define _ATOMICITY_H 1

#include <inttypes.h>

static inline int
__attribute__((unused))
exchange_and_add (volatile uint32_t *mem, int val)
{
	int tmp, result;

	__asm__ __volatile__(
		"/* Inline exchange and add */\n"
		"1:	ssrf	5\n"
		"	ld.w	%0, %3\n"
		"	add	%1, %0, %4\n"
		"	stcond	%2, %1\n"
		"	brne	1b"
		: "=&r"(result), "=&r"(tmp), "=m"(*mem)
		: "m"(*mem), "r"(val)
		: "cc", "memory");

	return result;
}

static inline void
__attribute__((unused))
atomic_add (volatile uin32_t *mem, int val)
{
	int result;

	__asm__ __volatile__(
		"/* Inline atomic add */\n"
		"1:	ssrf	5\n"
		"	ld.w	%0, %2\n"
		"	add	%0, %3\n"
		"	stcond	%2, %0\n"
		"	brne	1b"
		: "=&r"(result), "=m"(*mem)
		: "m"(*mem), "r"(val)
		: "cc", "memory");
}

static inline int
__attribute__((unused))
compare_and_swap(volatile long int *p, long int oldval, long int newval)
{
	long int result, tmp;

	__asm__ __volatile__(
		"/* Inline compare and swap */\n"
		"1:	ssrf	5\n"
		"	ld.w	%1, %3\n"
		"	cp.w	%1, %5\n"
		"	sreq	%0\n"
		"	brne	2f\n"
		"	stcond	%2, %4\n"
		"	brne	1b\n"
		"2:"
		: "=&r"(result), "=&r"(tmp), "=m"(*p)
		: "m"(*p), "r"(newval), "r"(oldval)
		: "cc", "memory");

	return result;
}

#endif /* atomicity.h */
